@@ -419,7 +419,7 @@ def identifyGaps(G : nx.Graph,
     if level < 0:
         assert 2 in motif_info.signature, 'signature does not contain two'
         positions  = sma.matchPositions(motif_info.signature, arities, roles)
-        level = positions[signature.index(2)]
+        level = positions[motif_info.signature.index(2)]
     contributions, edges = sma.edgeContributionList(G, 
                                                     *motif_info.signature, 
                                                     roles = positions, 
@@ -437,4 +437,32 @@ def identifyGaps(G : nx.Graph,
     else:
         # motif is a closed motif
         values_filtered = filter(lambda x : not(G.has_edge(*x[0])), values)
-    return sorted(values_filtered, key=operator.itemgetter(1), reverse=True)
\ No newline at end of file
+    return sorted(values_filtered, key=operator.itemgetter(1), reverse=True)
+
+def isClosedMotif(motif_identifier : str, level : int = -1) -> bool:
+    """
+    Queries the motif database and determines whether a motif is closed or open
+    with respect to a given level.
+    
+    See :py:meth:`sma.identifyGaps`.
+    
+    :param motif_identifier: motif identifier string
+    :param level: level (``sesType``) for the gap
+    :returns: true/false whether the motif is closed.
+    :raises NotImplementedError: if the motif database does not know..
+    """
+    arities, roles, motif = sma.parseMotifIdentifier(motif_identifier)
+    signature             = sma.multiSignature(arities)
+    motif_info            = sma.motifInfo(signature)
+    positions             = sma.matchPositions(motif_info.signature, arities, roles)
+    abstract_level        = positions.index(level) if level > 0 else motif_info.signature.index(2)
+    if abstract_level not in motif_info.projections:
+        raise NotImplementedError('closed/open relations not implemented for motifs with signature %s on abstract level %d'\
+                                  % (str(motif_info.signature), abstract_level))
+    for o, c in motif_info.projections[abstract_level]:
+        if c == motif:
+            return True
+        if o == motif:
+            return False
+    raise TypeError('unknown motif %s' % motif)
+    
\ No newline at end of file
