@@ -36,22 +36,116 @@ def expectedMultiMotifs(G : nx.Graph,
     :param roles: roles of the levels, cf. :py:class:`sma.MultiMotifClassificator`.
     :param kwargs: additional parameters for the functions, see above.
     """
-    signature = sma.classify._multiSignature(arities)
-    multis = [
-                # signature, ordered,  function
-                ([1, 2],    [1, 2],    sma.expected3Motifs),
-                ([2, 2],    [2, 2],    sma.expected4Motifs),
-                ([1, 1, 2], [1, 2, 1], sma.expected121Motifs),
-                ([1, 2, 2], [2, 2, 1], sma.expected221Motifs),
-                ([2, 2, 2,],[2, 2, 2], sma.expected222Motifs)
-            ]
-    for sig, sig_ordered, counter in multis:
-        if signature == sig:
-            positions = sma.classify._matchPositions(sig_ordered, arities, roles)
-            return counter(G, *positions, **kwargs)
-            
-    raise TypeError('No suitable function for computing expectations found')
+    signature  = sma.multiSignature(arities)
+    motif_info = sma.motifInfo(signature)
+    positions  = sma.matchPositions(motif_info.signature, arities, roles)
+    if motif_info.expectations is None:
+        raise NotImplementedError("expectations for %s-motifs not implemented" % str(motif_info.signature))
+    return motif_info.expectations(G, *positions, **kwargs)
+    
+def varMultiMotifs(G : nx.Graph,
+                        *arities, 
+                        roles = [],
+                        **kwargs):
+    """
+    Front-end function for several functions for computing motif variances. It supports
+    the same parameters for defining the motifs (arities, roles) as other multi-level
+    (counting) functions. Check the documentations of the respective functions.
+    The supported functions are:
+        
+        - :py:meth:`sma.var3Motifs`
+    
+    See the documentation for an exposition of the mathematical background.
+    
+    :param G: the SEN
+    :param arities: arities of the motifs, cf. :py:class:`sma.MultiMotifClassificator`.
+    :param roles: roles of the levels, cf. :py:class:`sma.MultiMotifClassificator`.
+    :param kwargs: additional parameters for the functions, see above.
+    """
+    signature  = sma.multiSignature(arities)
+    motif_info = sma.motifInfo(signature)
+    positions  = sma.matchPositions(motif_info.signature, arities, roles)
+    if motif_info.variances is None:
+        raise NotImplementedError("variances for %s-motifs not implemented" % str(motif_info.signature))
+    return motif_info.variances(G, *positions, **kwargs)
 
+def distributionMotifsAuto(G : nx.Graph, 
+                           *motifs : str, 
+                           model : str = sma.MODEL_ERDOS_RENYI,
+                           level : int = -1):
+    """
+    Front-end function for functions in sma which provide insight into the distribution
+    of motif counts in various baseline models.
+    
+    This function returns a tuple consisting of two dicts:
+        
+        1. a dict of partial results as requested using the motif identifiers. This
+           dict maps motif identifiers to tuples containing firstly the expectation
+           and secondly the variance of this motif.
+        2. a dict of the results of all motifs with same arities/roles as the motif
+           specified using the motif identifiers. This dict maps the heads of the
+           motif identifers to dicts mapping motif classes to tuples of expectations
+           and variances as above.
+           
+    Values which are not available in the selected model are given as ``None``.
+    
+    The following models (specify using parameter ``model``) are supported:
+        
+        - ``erdos_renyi`` (default) general Erdős-Rényi random graphs with fixed
+          edge probabilities for each subsystem equalling the densities of the 
+          given SEN, see documentation, cf. :py:const:`sma.MODEL_ERDOS_RENYI`.
+        - ``actors_choice`` edges in one specific level are chosen at random. The
+          remainder of the network is fixed. The edge probability in the variable
+          level is extracted from the given SEN. Use parameter ``level`` to specify
+          the ``sesType`` of the variable level. If :math:`-1` (default), then
+          the first entry equalling :math:`2` in the list of arities is selected,
+          cd. :py:const:`sma.MODEL_ACTORS_CHOICE`.
+    
+    :param G: the SEN
+    :param motifs: motif identifier strings
+    :param model: model to be used
+    :param level: additional parameter for model ``actors_choice``
+    :returns: tuple of two dicts as above
+    :raises TypeError: when the motif identifiers cannot be parsed
+    """
+    grouped_request = sma.groupMotifIdentifiers(*motifs)
+    partial_results = {}
+    total_results = {}
+    for head in grouped_request.keys():
+        arities, roles, _ = sma.parseMotifIdentifier(head)
+        if model == sma.MODEL_ERDOS_RENYI:
+            expectations = expectedMultiMotifs(G, *arities, roles = roles)
+            variances = None
+            try:
+                variances = varMultiMotifs(G, *arities, roles = roles)
+            except NotImplementedError:
+                # variance is not implemented for most motifs, so better pass
+                pass
+            total_results[head] = {k : (expectations[k] if expectations is not None else None, variances[k] if variances is not None else None) for k in expectations.keys()}
+        elif model == sma.MODEL_ACTORS_CHOICE:
+            total_results[head]  = distributionMotifsActorsChoice(G,
+                                                                    *arities,
+                                                                    roles = roles,
+                                                                    level = level,
+                                                                    array = False)
+        else:
+            raise NotImplementedError("model %s not supported" % model)
+        for motif in grouped_request[head]:
+            key = motif
+            if key not in total_results[head]:
+                try:
+                    if int(key) in total_results[head]:
+                        key = int(key)
+                    else:
+                        raise TypeError("Unrecognizable motif identifier %s[%s]." % (head, motif))
+                except ValueError:
+                    pass
+            try:
+                partial_results["%s[%s]" % (head, motif)] = total_results[head][key]
+            except KeyError:
+                raise TypeError("Unrecognizable motif identifier %s[%s]." % (head, motif))
+    return (partial_results, total_results)
+    
 def expected3Motifs(G : nx.Graph, 
                     level0, 
                     level1, 
@@ -349,6 +443,63 @@ def var3SMotifs(G : nx.Graph, **kwargs):
     """
     return var3Motifs(G, sma.NODE_TYPE_SOC, sma.NODE_TYPE_ECO, **kwargs)
 
+def distributionMotifsActorsChoice(G : nx.Graph, 
+                                   *arities, 
+                                   roles = [], 
+                                   level : int = -1,
+                                   array : bool = False):
+    """
+    Computes expectation and variances for Actor's Choice distribution of motifs.
+    See the documentation for a description of this model.
+    
+    If ``level = -1``, then the first entry equalling two in the signature of
+    the motif is taken as level.
+    
+    :param G: the SEN
+    :param arities: arities of the (multi-level) motif
+    :param level: level which is assumed to be variable in the Actor's Choice Model
+    :param array: whether the output is a tuple ``(e,v)`` where ``e`` is a vector
+        of expectations and ``v`` a vector of variances, or dict mapping motif names
+        to tuples of expectation and variance for this motif
+    :returns: properties of the distribution as tuple or dict (see ``array``)
+    """
+    signature  = sma.multiSignature(arities)
+    motif_info = sma.motifInfo(signature)
+    positions  = sma.matchPositions(motif_info.signature, arities, roles)
+    
+    if level < 0:
+        assert 2 in signature, 'signature does not contain two'
+        level = positions[signature.index(2)]
+    
+    contributions, _    = sma.edgeContributionList(G, *motif_info.signature, roles = positions, level = level)
+    p                   = sma.densityMatrix(G)[level,level]
+    sums                = numpy.sum(contributions, axis = 1)
+    squares             = numpy.sum(contributions**2, axis = 1)
+    expectations_open   = (1-p) * sums
+    expectations_closed = p * sums
+    variances           = p * (1-p) * squares
+    
+    # de-group open and closed motifs
+    indexer = {motif : i for i, motif in enumerate(motif_info.classes)}
+    abstract_level = positions.index(level)
+    if abstract_level not in motif_info.projections:
+        raise NotImplementedError('ActorsChoice not implemented for signature %s on level %d' \
+                                  % (str(motif_info.signature), level))
+    grouper = motif_info.projections[abstract_level]
+    e = numpy.zeros((len(indexer),))
+    v = numpy.zeros((len(indexer),))
+    for (op, cl), eo, ec, var in zip(grouper, expectations_open, expectations_closed, variances):
+        e[indexer[op]] = eo
+        e[indexer[cl]] = ec
+        v[indexer[op]] = v[indexer[cl]] = var
+    
+    if array:
+        return (e,v)
+    else:
+        return {k : (exp, var) for (k, exp, var) in zip(motif_info.classes, e, v)}
+
+
+# Statistical helper function
 def markovBound(value : numpy.ndarray, expectation : numpy.ndarray) -> numpy.ndarray:
     """
     Uses Markov's inequality to compute bounds for the probability that a random
