@@ -32,7 +32,7 @@ class MotifIterator:
     sources are tangible Python iterators, whereas conditions are filters to these
     iterators. 
     
-    Since in R binary operators are not available for complex types this module
+    Since in R binary operators are not available for complex types, this module
     provides synonymous functions for combining iterators which are callable in 
     R as well as in Python, cf. :py:meth:`sma.motifSet`, :py:meth:`sma.countAnyMotifs`.
     
@@ -255,7 +255,7 @@ class DenseMulti221Motifs(SourceMotifIterator):
         if optimize_top_adjacent:
             def __iterate(G):
                 for lower in sma.sesSubgraph(G, level2):
-                    top = sma.sesSubgraph(G, level0, G[lower])
+                    top = sma.sesSubgraph(G, level0, G[lower]) # top adjacent
                     middle = sma.sesSubgraph(G, level1, G[lower])
                     for mtop, mmiddle in itertools.product(itertools.combinations(top, 2), 
                                                            itertools.combinations(middle, 2)):
@@ -310,6 +310,8 @@ class MultiMotifs(SourceMotifIterator):
     yields :math:`a_i` nodes from level :math:`s_i` where :math:`(s_0, \dots, s_n)` denotes
     the array in ``subsystems``.
     
+    See also :py:class:`sma.MultiMotifsNormalized`.
+    
     :param G: the SEN
     :param arities: a list of integers representing the numbers of nodes which
         shall be taken from the different levels of the SEN.
@@ -339,7 +341,31 @@ class MultiMotifs(SourceMotifIterator):
             product = map(tuple, map(itertools.chain.from_iterable, product))
         super().__init__(G, product)
     def __str__(self):
-        return ('MultiMotifs[' + ','.join(['%d'] * len(self.arities)) + ']') % self.arities
+        return ('MultiMotifs%s') % str(self.arities)
+
+class MultiMotifsNormalized(MultiMotifs):
+    """
+    Source for multi-motifs with entries in the order specified by the signature
+    of the motif. For example, 3-motifs are always given as tuples (A, B, B)
+    regardless of the ``sesType``'s of the levels that A and B come from.
+    
+    Position matching is used to identify the relavent levels.
+    Extends :py:class:`sma.MultiMotifs`.
+    
+    :param G: the SEN
+    :param arities: list of arities
+    :param roles: roles, optionally
+    :param motif_info: a subclass of :py:class:`sma.MotifInfo`, optionally
+    """
+    def __init__(self, G: nx.Graph, 
+                 *arities : int, 
+                 roles = [],
+                 motif_info = None):
+        if motif_info is None:
+            signature  = sma.multiSignature(arities)
+            motif_info = sma.motifInfo(signature)
+        positions  = sma.matchPositions(motif_info.signature, arities, roles)
+        super().__init__(G, *motif_info.signature, flatten = True, subsystems = positions)
 
 class hasProperty(ConditionMotifIterator):
     """
@@ -431,52 +457,52 @@ class hasNode(ConditionMotifIterator):
     def __str__(self):
         return 'hasNode[%s]' % self.node
         
-class is4Type(ConditionMotifIterator):
+class is4Class(ConditionMotifIterator):
     """
-    Condition for 4-motifs modelling that the motif is of certain types, cf.
+    Condition for 4-motifs modelling that the motif is of certain classes, cf.
     :py:meth:`sma.classify4Motif`.
     
     This condition works only for 4-motifs.
     
-    :param typ: types of the motifs to look for, e.g. 'II.C'.
+    :param typ: classes of the motifs to look for, e.g. 'II.C'.
     """
     def __init__(self, *typ):
         self.typ = typ
         super().__init__(lambda G, motif : sma.classify4Motif(G, motif) in typ)
     def __str__(self):
-        return 'has4Type[%s]' % self.typ
+        return 'has4Class[%s]' % self.typ
         
-class is3Type(ConditionMotifIterator):
+class is3Class(ConditionMotifIterator):
     """
-    Condition for 3-motifs modelling that the motif is of certain types, cf.
+    Condition for 3-motifs modelling that the motif is of certain classes, cf.
     :py:meth:`sma.classify3Motif`.
     
     This condition works only for 3-motifs.
     
-    :param typ: types of the motifs to look for, e.g. 'II.C'.
+    :param typ: classes of the motifs to look for, e.g. 'II.C'.
     """
     def __init__(self, *typ):
         self.typ = typ
         super().__init__(lambda G, motif : sma.classify3Motif(G, motif) in typ)
     def __str__(self):
-        return 'has3Type[%s]' % self.typ
+        return 'has3Class[%s]' % self.typ
 
-class isType(ConditionMotifIterator):
+class isClass(ConditionMotifIterator):
     """
-    Condition for any sort of motif modelling that a motif is of a certain type.
+    Condition for any sort of motif modelling that a motif is of a certain class.
     Provide a :py:class:`sma.MotifClassificator` as first parameter. The classification
-    generated by this classificator will be used to match the given types.
+    generated by this classificator will be used to match the given classes.
     
     :param classificator: a :py:class:`sma.MotifClassificator` for classifying
         the motifs
-    :param typ: one or several types
+    :param typ: one or several classes
     """
     def __init__(self, classificator : sma.MotifClassificator, *typ):
         self.typ = typ
         self.classificator = classificator
         super().__init__(lambda G, motif: classificator(motif) in typ)
     def __str__(self):
-        return 'hasType[%s=%s]' % (str(self.classificator), self.typ)
+        return 'hasClass[%s=%s]' % (str(self.classificator), self.typ)
 
 class matchesPropertyPattern4Motif(ConditionMotifIterator):
     """
