@@ -172,4 +172,16 @@ def identifyGapsR(G : nx.Graph,
     df = pandas.DataFrame(map(lambda x: (*x[0],x[1]), result), 
                           columns=['vertex0', 'vertex1', 'contribution'])
     return df
-    
\ No newline at end of file
+
+def isClosedMotifR(motif_identifier : str, level : int = -1) -> bool:
+    """
+    Wrapper function for :py:meth:`sma.isClosedMotif`. Optimized for the reticulate
+    R interface.
+    
+    :param motif_identifier: motif identifier
+    :param level: level
+    :returns: whether the motif is closed with respect to the specified level
+    :raises NotImplementedError: if open/closed relations are not implemented for
+        this motif and level
+    """
+    return sma.isClosedMotif(motif_identifier, int(level))
\ No newline at end of file
